@@ -2,6 +2,7 @@ package org.gcube.portlets.user.td.gwtservice.shared.monitor;
 
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
+import java.util.List;
 
 import javax.servlet.http.HttpSession;
 
@@ -9,18 +10,22 @@ import org.gcube.application.framework.core.session.ASLSession;
 import org.gcube.data.analysis.tabulardata.commons.utils.AuthorizationProvider;
 import org.gcube.data.analysis.tabulardata.commons.utils.AuthorizationToken;
 import org.gcube.data.analysis.tabulardata.commons.webservice.types.TaskStatus;
+import org.gcube.data.analysis.tabulardata.model.metadata.common.TableDescriptorMetadata;
 import org.gcube.data.analysis.tabulardata.model.metadata.table.DatasetViewTableMetadata;
 import org.gcube.data.analysis.tabulardata.model.metadata.table.ExportMetadata;
 import org.gcube.data.analysis.tabulardata.model.table.Table;
+import org.gcube.data.analysis.tabulardata.model.table.TableId;
 import org.gcube.data.analysis.tabulardata.service.TabularDataService;
 import org.gcube.data.analysis.tabulardata.service.impl.TabularDataServiceFactory;
 import org.gcube.data.analysis.tabulardata.service.operation.Job;
 import org.gcube.data.analysis.tabulardata.service.operation.Task;
+import org.gcube.data.analysis.tabulardata.service.operation.TaskResult;
 import org.gcube.data.analysis.tabulardata.service.operation.ValidationJob;
 import org.gcube.data.analysis.tabulardata.service.tabular.TabularResource;
 import org.gcube.data.analysis.tabulardata.service.tabular.TabularResourceId;
 import org.gcube.portlets.user.td.gwtservice.server.SessionUtil;
 import org.gcube.portlets.user.td.gwtservice.server.storage.FilesStorage;
+import org.gcube.portlets.user.td.gwtservice.server.trservice.JobClassifierMap;
 import org.gcube.portlets.user.td.gwtservice.server.trservice.TabularResourceTypeMap;
 import org.gcube.portlets.user.td.gwtservice.server.trservice.TaskStateMap;
 import org.gcube.portlets.user.td.gwtservice.server.trservice.WorkerStateMap;
@@ -29,12 +34,11 @@ import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTServiceExcept
 import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTSessionExpiredException;
 import org.gcube.portlets.user.td.gwtservice.shared.json.JSONExportSession;
 import org.gcube.portlets.user.td.gwtservice.shared.task.JobS;
-import org.gcube.portlets.user.td.gwtservice.shared.task.State;
+import org.gcube.portlets.user.td.gwtservice.shared.task.JobSClassifier;
 import org.gcube.portlets.user.td.gwtservice.shared.task.TaskS;
 import org.gcube.portlets.user.td.gwtservice.shared.task.TaskWrapper;
 import org.gcube.portlets.user.td.gwtservice.shared.task.ValidationsJobS;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.TabResource;
-import org.gcube.portlets.user.td.gwtservice.shared.tr.table.metadata.TabExportMetadata;
 import org.gcube.portlets.user.td.widgetcommonevent.shared.TRId;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -46,19 +50,21 @@ import org.slf4j.LoggerFactory;
  * 
  */
 public class OperationMonitorCreator {
-	protected static Logger logger = LoggerFactory
+	private static final String SECURITY_EXCEPTION_RIGHTS = "Security exception, you don't have the required rights!";
+
+	private static Logger logger = LoggerFactory
 			.getLogger(OperationMonitorCreator.class);
 
-	protected static SimpleDateFormat sdf = new SimpleDateFormat(
-			"yyyy-MM-dd HH:mm");
+	//private static SimpleDateFormat sdf = new SimpleDateFormat(
+	//		"yyyy-MM-dd HH:mm");
 
-	protected static SimpleDateFormat sdfDate = new SimpleDateFormat(
-			"yyyy-MM-dd");
+	@SuppressWarnings("unused")
+	private static SimpleDateFormat sdfDate = new SimpleDateFormat("yyyy-MM-dd");
 
-	protected HttpSession session;
-	protected TaskWrapper taskWrapper;
-	protected OperationMonitorSession operationMonitorSession;
-	protected ASLSession aslSession;
+	private HttpSession session;
+	private TaskWrapper taskWrapper;
+	private OperationMonitorSession operationMonitorSession;
+	private ASLSession aslSession;
 
 	/**
 	 * 
@@ -101,9 +107,9 @@ public class OperationMonitorCreator {
 				Task task = taskWrapper.getTask();
 				task.abort();
 				TaskS taskS = createTaskS();
-				taskS.setState(State.ABORTED);
+				//taskS.setState(State.ABORTED);
 				operationMonitor.setTask(taskS);
-				operationMonitor.setAbort(true);
+				//operationMonitor.setAbort(true);
 				SessionUtil.setStartedTask(session, taskWrapper);
 				postOperation(operationMonitor);
 			} else {
@@ -143,7 +149,7 @@ public class OperationMonitorCreator {
 		return operationMonitor;
 	}
 
-	protected TaskS createTaskS() throws  TDGWTServiceException {
+	protected TaskS createTaskS() throws TDGWTServiceException {
 		TaskS taskS = new TaskS();
 
 		ArrayList<JobS> jobSList = new ArrayList<JobS>();
@@ -161,9 +167,12 @@ public class OperationMonitorCreator {
 				validationsJobS.add(validationJ);
 				j++;
 			}
+			
+			JobSClassifier jobClassifier=JobClassifierMap.map(job.getJobClassifier());
+			
 
 			JobS jobS = new JobS(String.valueOf(i), job.getProgress(),
-					job.getHumaReadableStatus(), job.getDescription(),
+					job.getHumaReadableStatus(), jobClassifier, job.getDescription(),
 					WorkerStateMap.map(job.getStatus()), job.getErrorMessage(),
 					validationsJobS);
 
@@ -173,17 +182,36 @@ public class OperationMonitorCreator {
 		}
 
 		try {
+
+			ArrayList<TRId> collateralTRIds = new ArrayList<TRId>();
+			TaskResult taskResult = taskWrapper.getTask().getResult();
+			if (taskResult != null) {
+				List<TableId> collaterals = taskResult.getCollateralTables();
+				for (TableId tId : collaterals) {
+					String tabulRId = retrieveTabularResourceIdFromTable(tId);
+					TRId tabularRId = new TRId(tabulRId);
+					tabularRId.setTableId(String.valueOf(tId.getValue()));
+					collateralTRIds.add(tabularRId);
+				}
+
+			}
+
 			taskS = new TaskS(taskWrapper.getTask().getId().getValue(),
 					taskWrapper.getTask().getProgress(),
 					TaskStateMap.map(taskWrapper.getTask().getStatus()),
 					taskWrapper.getTask().getErrorCause(), taskWrapper
 							.getTask().getSubmitter(), taskWrapper.getTask()
 							.getStartTime(),
-					taskWrapper.getTask().getEndTime(), jobSList);
+					taskWrapper.getTask().getEndTime(), jobSList,
+					collateralTRIds, String.valueOf(taskWrapper.getTask()
+							.getTabularResourceId().getValue()));
 		} catch (Throwable e) {
-			logger.error("error retrieving information about the task, "+e.getLocalizedMessage());
+			logger.error("error retrieving information about the task, "
+					+ e.getLocalizedMessage());
 			e.printStackTrace();
-			throw new  TDGWTServiceException("error retrieving information about the task, "+e.getLocalizedMessage());
+			throw new TDGWTServiceException(
+					"error retrieving information about the task, "
+							+ e.getLocalizedMessage());
 		}
 		logger.debug("Retrieved task information");
 		return taskS;
@@ -215,6 +243,7 @@ public class OperationMonitorCreator {
 		case GENERATING_VIEW:
 			break;
 		case ABORTED:
+			operationMonitor.setAbort(true);
 			break;
 		case STOPPED:
 			logger.debug("Task Result:" + taskWrapper.getTask().getResult());
@@ -232,9 +261,9 @@ public class OperationMonitorCreator {
 		TRId trId;
 		TabResource tabResource;
 		Table table;
-		TabExportMetadata trExportMetadata;
+		//TabExportMetadata trExportMetadata;
 		ExportMetadata exportMetadata;
-		
+
 		switch (taskWrapper.getOperationId()) {
 		case CSVExport:
 			table = taskWrapper.getTask().getResult().getPrimaryTable();
@@ -244,12 +273,12 @@ public class OperationMonitorCreator {
 
 			operationMonitor.setTrId(SessionUtil.getTRId(session));
 
-			trExportMetadata = new TabExportMetadata();
+			/*trExportMetadata = new TabExportMetadata();
 			trExportMetadata.setUrl(exportMetadata.getUri());
 			trExportMetadata.setDestinationType(exportMetadata
 					.getDestinationType());
 			trExportMetadata.setExportDate(sdf.format(exportMetadata
-					.getExportDate()));
+					.getExportDate()));*/
 
 			saveCSVExportInDestination(exportMetadata);
 			break;
@@ -267,24 +296,23 @@ public class OperationMonitorCreator {
 			logger.debug("ExportMetadata: " + exportMetadata);
 
 			operationMonitor.setTrId(SessionUtil.getTRId(session));
-			
-			trExportMetadata = new TabExportMetadata();
+
+			/*trExportMetadata = new TabExportMetadata();
 			trExportMetadata.setUrl(exportMetadata.getUri());
 			trExportMetadata.setDestinationType(exportMetadata
 					.getDestinationType());
 			trExportMetadata.setExportDate(sdf.format(exportMetadata
-					.getExportDate()));
+					.getExportDate()));*/
 
 			saveJSONExportInDestination(exportMetadata);
-			break;	
-		case CSVImport:	
+			break;
+		case CSVImport:
 			trId = new TRId();
 			trId.setId(taskWrapper.getTrId().getId());
 			trId = retrieveTabularResourceBasicData(trId);
 
 			operationMonitor.setTrId(trId);
 			tabResource = SessionUtil.getCSVImportTabResource(session);
-			logger.debug("CurrentTabResource :" + tabResource);
 			if (tabResource != null) {
 				tabResource.setTrId(trId);
 				SessionUtil.setTabResource(session, tabResource);
@@ -293,17 +321,17 @@ public class OperationMonitorCreator {
 				tabResource.setTrId(trId);
 				SessionUtil.setTabResource(session, tabResource);
 			}
+			logger.debug("CurrentTabResource set:" + tabResource);
 			SessionUtil.setTRId(session, trId);
-			SessionUtil.addToCurrentTabularResourcesOpen(session,tabResource);
+			SessionUtil.addToCurrentTabularResourcesOpen(session, tabResource);
 			break;
-		case SDMXImport:	
+		case SDMXImport:
 			trId = new TRId();
 			trId.setId(taskWrapper.getTrId().getId());
 			trId = retrieveTabularResourceBasicData(trId);
 
 			operationMonitor.setTrId(trId);
 			tabResource = SessionUtil.getSDMXImportTabResource(session);
-			logger.debug("CurrentTabResource :" + tabResource);
 			if (tabResource != null) {
 				tabResource.setTrId(trId);
 				SessionUtil.setTabResource(session, tabResource);
@@ -312,18 +340,21 @@ public class OperationMonitorCreator {
 				tabResource.setTrId(trId);
 				SessionUtil.setTabResource(session, tabResource);
 			}
+			logger.debug("CurrentTabResource set:" + tabResource);
 			SessionUtil.setTRId(session, trId);
-			SessionUtil.addToCurrentTabularResourcesOpen(session,tabResource);
-			break;	
-			
-		default:
+			SessionUtil.addToCurrentTabularResourcesOpen(session, tabResource);
+			break;
+		case Clone:
 			trId = new TRId();
 			trId.setId(taskWrapper.getTrId().getId());
 			trId = retrieveTabularResourceBasicData(trId);
-
+			operationMonitor.setTrId(trId);
+			break;
+		default:
+			trId = new TRId(taskWrapper.getTrId().getId());
+			trId = retrieveTabularResourceBasicData(trId);
 			operationMonitor.setTrId(trId);
 			tabResource = SessionUtil.getTabResource(session);
-			logger.debug("CurrentTabResource :" + tabResource);
 			if (tabResource != null) {
 				tabResource.setTrId(trId);
 				SessionUtil.setTabResource(session, tabResource);
@@ -332,6 +363,7 @@ public class OperationMonitorCreator {
 				tabResource.setTrId(trId);
 				SessionUtil.setTabResource(session, tabResource);
 			}
+			logger.debug("CurrentTabResource set:" + tabResource);
 			SessionUtil.setTRId(session, trId);
 			break;
 
@@ -355,8 +387,9 @@ public class OperationMonitorCreator {
 				logger.error("Tabular Resource Null [id=" + trId.getId() + "]");
 				throw new TDGWTServiceException("Tabular Resource Null " + trId);
 			}
-			//logger.debug("Retrieve Tabular Resource Basic Data: "+tr);
+			// logger.debug("Retrieve Tabular Resource Basic Data: "+tr);
 
+			
 			Table table = service.getLastTable(tabularResourceId);
 			if (table == null) {
 				logger.error("Last Table Null");
@@ -378,14 +411,18 @@ public class OperationMonitorCreator {
 
 			TRId newTRId;
 			if (viewTable == null) {
-				newTRId = new TRId(String.valueOf(tr.getId().getValue()),
-						TabularResourceTypeMap.map(tr.getTabularResourceType()), tr.getTableType(), String.valueOf(table.getId()
-										.getValue()), table.getTableType().getName());
+				newTRId = new TRId(
+						String.valueOf(tr.getId().getValue()),
+						TabularResourceTypeMap.map(tr.getTabularResourceType()),
+						tr.getTableType(), String.valueOf(table.getId()
+								.getValue()), table.getTableType().getName());
 
 			} else {
-				newTRId = new TRId(String.valueOf(tr.getId().getValue()),
-						TabularResourceTypeMap.map(tr.getTabularResourceType()), tr.getTableType(), String.valueOf(viewTable.getId()
-										.getValue()), viewTable.getTableType()
+				newTRId = new TRId(
+						String.valueOf(tr.getId().getValue()),
+						TabularResourceTypeMap.map(tr.getTabularResourceType()),
+						tr.getTableType(), String.valueOf(viewTable.getId()
+								.getValue()), viewTable.getTableType()
 								.getName(), String.valueOf(table.getId()
 								.getValue()), true);
 
@@ -398,8 +435,7 @@ public class OperationMonitorCreator {
 			throw e;
 		} catch (SecurityException e) {
 			e.printStackTrace();
-			throw new TDGWTServiceException(
-					"Security exception, you haven't rights!");
+			throw new TDGWTServiceException(SECURITY_EXCEPTION_RIGHTS);
 		} catch (Throwable e) {
 			e.printStackTrace();
 			throw new TDGWTServiceException("Error Retrieving Basic Data: "
@@ -430,16 +466,18 @@ public class OperationMonitorCreator {
 			boolean end = SessionUtil.getCSVExportEnd(session);
 			if (end == false) {
 				SessionUtil.setCSVExportEnd(session, true);
-				FilesStorage storage = new FilesStorage();
-				logger.debug("Create Item On Workspace: [ uri: "
+				
+				logger.debug("Create Item On Workspace: [ storageId: "
 						+ exportMetadata.getUri() + " ,user: " + user
 						+ " ,fileName: " + exportSession.getFileName()
 						+ " ,fileDescription: "
 						+ exportSession.getFileDescription()
 						+ " ,mimetype: text/csv" + " ,folder: "
 						+ exportSession.getItemId() + "]");
-				storage.createItemOnWorkspace(exportMetadata.getUri(), user,
-						exportSession.getFileName(),
+				
+				FilesStorage storage = new FilesStorage();
+				storage.createItemOnWorkspaceByStorageId(exportMetadata.getUri(), user,
+						exportSession.getFileName()+".csv",
 						exportSession.getFileDescription(), "text/csv",
 						exportSession.getItemId());
 
@@ -452,8 +490,7 @@ public class OperationMonitorCreator {
 					"Error in export csv: no destination present");
 		}
 	}
-	
-	
+
 	/**
 	 * Save export json data on Workspace
 	 * 
@@ -477,16 +514,19 @@ public class OperationMonitorCreator {
 			boolean end = SessionUtil.getJSONExportEnd(session);
 			if (end == false) {
 				SessionUtil.setJSONExportEnd(session, true);
-				FilesStorage storage = new FilesStorage();
-				logger.debug("Create Item On Workspace: [ uri: "
+				
+				logger.debug("Create Item On Workspace: [ storageId: "
 						+ exportMetadata.getUri() + " ,user: " + user
 						+ " ,fileName: " + exportSession.getFileName()
 						+ " ,fileDescription: "
 						+ exportSession.getFileDescription()
 						+ " ,mimetype: application/json" + " ,folder: "
 						+ exportSession.getItemId() + "]");
-				storage.createItemOnWorkspace(exportMetadata.getUri(), user,
-						exportSession.getFileName(),
+				
+				FilesStorage storage = new FilesStorage();
+				
+				storage.createItemOnWorkspaceByStorageId(exportMetadata.getUri(), user,
+						exportSession.getFileName()+".json",
 						exportSession.getFileDescription(), "application/json",
 						exportSession.getItemId());
 
@@ -500,4 +540,29 @@ public class OperationMonitorCreator {
 		}
 	}
 
+	private String retrieveTabularResourceIdFromTable(TableId tableId)
+			throws TDGWTServiceException {
+		try {
+			AuthorizationProvider.instance.set(new AuthorizationToken(
+					aslSession.getUsername(), aslSession.getScope()));
+			TabularDataService service = TabularDataServiceFactory.getService();
+
+			Table table = service.getTable(tableId);
+
+			if (table.contains(TableDescriptorMetadata.class)) {
+				TableDescriptorMetadata tdm = table
+						.getMetadata(TableDescriptorMetadata.class);
+				return String.valueOf(tdm.getRefId());
+			} else {
+				throw new TDGWTServiceException(
+						"No TableDescriptorMetadata present in tableId: "
+								+ tableId);
+			}
+
+		} catch (Exception e) {
+			throw new TDGWTServiceException(e.getLocalizedMessage());
+		}
+
+	}
+
 }
